@@ -22,8 +22,10 @@
 ##' \deqn{\Phi^{-1}(F_1(X_1)-U_1 p_1(X_1))\:,...,\:\Phi^{-1}(F_n(X_n)-U_n p_n(X_n))}
 ##' These are also iid standard normal.
 ##'
-##' The user must specify one of the following methods to calculate
-##' the residuals:
+##' @section Choosing the method:
+##' The user must specify the method used to calculate the residuals - see detailed list of method descriptions below.
+##' We note that all the methods are based on approximations. While the default 'oneStepGaussianoffMode' often represents a good compromise between accuracy and speed, it cannot be assumed to work well for all model classes.
+##' As a rule of thumb, if in doubt whether a method is accurate enough, you should always compare with the 'oneStepGeneric' which is considered the most accurate of the available methods.
 ##' \describe{
 ##' \item{method="fullGaussian"}{
 ##' This method assumes that the joint distribution of data \emph{and}
@@ -63,6 +65,10 @@
 ##'
 ##' The new data vector (\code{keep}) need not be passed from \R. It
 ##' automatically becomes a copy of \code{x} filled with ones.
+##'
+##' Some extra parameters are essential for the method.
+##' Pay special attention to the integration domain which must be specified either via \code{range} (continuous case) or \code{discreteSupport} (discrete case).
+##' It may be useful to look at the one step predictive distributions on either log scale (\code{trace=2}) or natural scale (\code{trace=3}) to determine which alternative methods might be appropriate.
 ##' }
 ##' \item{method="oneStepGaussian"}{
 ##' This is a special case of the generic method where the one step
@@ -84,9 +90,9 @@
 ##'     DATA_VECTOR(x);
 ##'     DATA_VECTOR_INDICATOR(keep, x);
 ##'     ...
-##'     nll -= keep(i) * dnorm(x(i), u(i), 0.0, true);
-##'     nll -= keep.cdf_lower(i) * log( pnorm(x(i), u(i), 0.0) );
-##'     nll -= keep.cdf_upper(i) * log( 1.0 - pnorm(x(i), u(i), 0.0) );
+##'     nll -= keep(i) * dnorm(x(i), u(i), sd(i), true);
+##'     nll -= keep.cdf_lower(i) * log( pnorm(x(i), u(i), sd(i)) );
+##'     nll -= keep.cdf_upper(i) * log( 1.0 - pnorm(x(i), u(i), sd(i)) );
 ##'     ...
 ##' }
 ##'
@@ -105,15 +111,25 @@
 ##' @param conditional Index vector of observations that are fixed during OSA. By default the empty set.
 ##' @param discrete Are observations discrete? (assumed FALSE by default)
 ##' @param discreteSupport Possible outcomes of discrete distribution (\code{method="oneStepGeneric"} only).
-##' @param range Possible range of the observations.
-##' @param seed Randomization seed (discrete case only). If \code{NULL} the RNG seed is untouched by this routine.
+##' @param range Possible range of the observations. (\code{method="oneStepGeneric"} only).
+##' @param seed Randomization seed (discrete case only). If \code{NULL} the RNG seed is untouched by this routine (recommended for simulation studies).
 ##' @param parallel Run in parallel using the \code{parallel} package?
-##' @param trace Trace progress?
-##' @param reverse Do calculations in opposite order to improve stability ? (currently enabled by default for \code{oneStepGaussianOffMode} method only)
+##' @param trace Logical; Trace progress? More options available for \code{method="oneStepGeneric"} - see details.
+##' @param reverse Do calculations in opposite order to improve stability? (currently enabled by default for \code{oneStepGaussianOffMode} method only)
 ##' @param ... Control parameters for OSA method
 ##' @return \code{data.frame} with OSA \emph{standardized} residuals
-##' in column \code{residual}. Depending on the method the output may
-##' also include OSA expected observation in column \code{mean}.
+##' in column \code{residual}. In addition, depending on the method, the output
+##' includes selected characteristics of the predictive distribution (current row) given past observations (past rows), notably the \emph{conditional}
+##' \describe{
+##' \item{mean}{Expectation of the current observation}
+##' \item{sd}{Standard deviation of the current observation}
+##' \item{Fx}{CDF at current observation}
+##' \item{px}{Density at current observation}
+##' \item{nll}{Negative log density at current observation}
+##' \item{nlcdf.lower}{Negative log of the lower CDF at current observation}
+##' \item{nlcdf.upper}{Negative log of the upper CDF at current observation}
+##' }
+##' \emph{given past observations}.
 ##' @examples
 ##' ######################## Gaussian case
 ##' runExample("simple")
@@ -207,9 +223,15 @@ oneStepPredict <- function(obj,
     ## Find randomeffects character
     args$random <- names.random
     args$regexp <- FALSE
-    ## Move data$name to parameter$name
-    args$parameters[observation.name] <- args$data[observation.name]
-    args$data[observation.name] <- NULL
+    ## Do we need to change, or take derivatives wrt., observations?
+    ## (search the code to see if a method uses "observation(k,y)" or
+    ## just "observation(k)").
+    obs.are.variables <- (method != "cdf")
+    ## Move data$name to parameter$name if necessary
+    if (obs.are.variables) {
+        args$parameters[observation.name] <- args$data[observation.name]
+        args$data[observation.name] <- NULL
+    }
     ## Make data.term.indicator in parameter list
     if(!is.null(data.term.indicator)){
         one <- rep(1, length(obs))
@@ -219,6 +241,15 @@ oneStepPredict <- function(obj,
         } else {
             args$parameters[[data.term.indicator]] <- cbind(one)
         }
+        ## Set attribute to tell the order of observations
+        ord <- rep(NA, length(obs))
+        ord[conditional] <- 0 ## First (out of bounds)
+        ord[subset] <- seq_along(subset)
+        ord[unconditional] <- length(obs) + 1 ## Never (out of bounds)
+        if (any(is.na(ord))) {
+            stop("Failed to determine the order of obervations")
+        }
+        attr(args$parameters[[data.term.indicator]], "ord") <- as.double(ord - 1)
     }
     ## Pretend these are *not observed*:
     if(length(unconditional)>0){
@@ -242,7 +273,8 @@ oneStepPredict <- function(obj,
         factor(fac)
     }
     map <- list()
-    map[[observation.name]] <- makeFac(obs)
+    if (obs.are.variables)
+        map[[observation.name]] <- makeFac(obs)
     if(!is.null(data.term.indicator)){
         map[[data.term.indicator]] <- makeFac(args$parameters[[data.term.indicator]])
     }
@@ -526,11 +558,15 @@ oneStepPredict <- function(obj,
     if(method == "cdf"){
         p <- newobj$par
         newobj$fn(p) ## Test eval
+        newobj$env$random.start <- expression(last.par[random])
         cdf <- function(k){
             tracefun(k)
             nll <- newobj$fn(observation(k))
+            lp <- newobj$env$last.par
             nlcdf.lower <- newobj$fn(observation(k, lower.cdf = TRUE))
+            newobj$env$last.par <- lp ## restore
             nlcdf.upper <- newobj$fn(observation(k, upper.cdf = TRUE))
+            newobj$env$last.par <- lp ## restore
             c(nll=nll, nlcdf.lower=nlcdf.lower, nlcdf.upper=nlcdf.upper)
         }
         pred <- applyMethod(cdf)
